/*
 *  Copyright (C) 2017 by Friedrich W. H. Kossebau <kossebau@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include "ktexteditorpreviewview.h"

#include "ktexteditorpreviewplugin.h"
#include "previewwidget.h"

// KF
#include <KLocalizedString>
#include <KTextEditor/MainWindow>

// Qt
#include <QIcon>
#include <QLayout>

using namespace KTextEditorPreview;

KTextEditorPreviewView::KTextEditorPreviewView(KTextEditorPreviewPlugin *plugin, KTextEditor::MainWindow *mainWindow)
    : QObject(mainWindow)
{
    Q_UNUSED(plugin);

    m_toolView = mainWindow->createToolView(plugin, QStringLiteral("ktexteditorpreviewplugin"), KTextEditor::MainWindow::Right, QIcon::fromTheme(QStringLiteral("document-preview")), i18n("Preview"));

    // add preview widget
    m_previewView = new PreviewWidget(plugin, mainWindow, m_toolView.data());
    m_toolView->layout()->setContentsMargins(0, 0, 0, 0);
    m_toolView->layout()->addWidget(m_previewView);
    m_toolView->addActions(m_previewView->actions());
}

KTextEditorPreviewView::~KTextEditorPreviewView()
{
    delete m_toolView;
}

void KTextEditorPreviewView::readSessionConfig(const KConfigGroup &config)
{
    m_previewView->readSessionConfig(config);
}

void KTextEditorPreviewView::writeSessionConfig(KConfigGroup &config)
{
    m_previewView->writeSessionConfig(config);
}
